// Copyright 2012 Thomas Loy.
//
// This file is part of LastLog.
//
// LastLog is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// LastLog is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LastLog.  If not, see <http://www.gnu.org/licenses/>.

package edu.self.lastLog;

import org.bukkit.ChatColor;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.plugin.java.JavaPlugin;

public class LastLogPlugin extends JavaPlugin implements Listener
{
        final public static int PAGE_LENGTH = 10; // how many lines per page?
        final public static String PLAYER_NAME_COLOR = "" + ChatColor.GOLD;
        final public static String HEADER_COLOR = "" + ChatColor.AQUA;
        final public static String DATE_COLOR = "" + ChatColor.YELLOW;
        final public static String UNKNOWN_COLOR = "" + ChatColor.GRAY;
        final public static String ERROR_COLOR = "" + ChatColor.RED;
        final public static String NOTIFY_PERMISSION = "lastlog.notify";
        private Command firstlogCommand = null;
        private Command lastlogCommand = null;
        private Command loginfoCommand = null;
        private PlayerList firstlogList;
        private PlayerList lastlogList;
        
        @Override
        public void onEnable() {
                firstlogCommand = getCommand("firstlog");
                lastlogCommand = getCommand("lastlog");
                loginfoCommand = getCommand("loginfo");
                getServer().getPluginManager().registerEvents(this, this);
                // Apparently what takes the most time are the following I/O heavy instructions.
                // Hence, their output will be cached.
                OfflinePlayer[] players = getServer().getOfflinePlayers();
                firstlogList = new PlayerList(players, false);
                lastlogList = new PlayerList(players, true);
        }

        @Override
        public void onDisable() {
        }
        
        @Override
        public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
                if (command == loginfoCommand) return executeLoginfo(sender, label, args);
                if (command == firstlogCommand) return executeFirstlog(sender, label, args);
                if (command == lastlogCommand) return executeLastlog(sender, label, args);
                return false;
        }

        @EventHandler
        public void onPlayerJoin(PlayerJoinEvent event) {
                Player player = event.getPlayer();
                String name = player.getName();
                long first = player.getFirstPlayed();
                long last = player.getLastPlayed();
                // on first login, getLastPlayed() returns 0.
                // 1970-01-01 is over, so let's not use it.
                if (last == 0l) last = first;
                lastlogList.set(name, last);
                if (!player.hasPlayedBefore()) {
                        firstlogList.set(name, first);
                        for (Player rec : getServer().getOnlinePlayers()) {
                                if (rec.hasPermission(NOTIFY_PERMISSION)) {
                                        rec.sendMessage(UNKNOWN_COLOR + name + HEADER_COLOR + " has logged in for the first time");
                                }
                        }
                        System.out.println(name + " has logged in for the first time");
                }
        }

        // the loginfo command
        public boolean executeLoginfo(CommandSender sender, String label, String[] args) {
                if (args.length == 0) return false;
                for (String arg : args) {
                        // try match exact name with any known player
                        OfflinePlayer player = getServer().getOfflinePlayer(arg);
                        // if none is found, try and be smart
                        if (!player.hasPlayedBefore()) {
                                player = getServer().getPlayer(arg);
                        }
                        if (player == null) {
                                sender.sendMessage("Player " + UNKNOWN_COLOR + arg + ChatColor.WHITE + " is unknown");
                        } else {
                                String name = player.getName();
                                long first = player.getFirstPlayed();
                                long last = player.getLastPlayed();
                                sender.sendMessage("Player " + PLAYER_NAME_COLOR + name);
                                sender.sendMessage(DATE_COLOR + new LastLogDate(first) + ChatColor.WHITE + " First Login");
                                sender.sendMessage(DATE_COLOR + new LastLogDate(last) + ChatColor.WHITE + " Last Login");
                        }
                }
                return true;
        }

        /* the lastlog command */
        public boolean executeLastlog(CommandSender sender, String label, String[] args) {
                return executeSomelog(sender, label, args, true);
        }

        /* the firstlog command */
        public boolean executeFirstlog(CommandSender sender, String label, String[] args) {
                return executeSomelog(sender, label, args, false);
        }

        /* handle common tasks of firstlog and lastlog */
        public boolean executeSomelog(CommandSender sender, String label, String[] args, boolean lastlog) {
                if (args.length > 1) {
                        return false;
                }
                int pageNumber = 0;
                if (args.length == 1) {
                        try {
                                pageNumber = Integer.parseInt(args[0]) - 1;
                        } catch (IllegalArgumentException iae) {
                                return false;
                        }
                }
                if (pageNumber < 0) return false;
                PlayerList playerList;
                if (lastlog) playerList = lastlogList;
                else playerList = firstlogList;
                playerList.sort();
                playerList.displayPage(pageNumber, lastlog, sender);
                return true;
        }
}
