// Copyright 2012 Thomas Loy.
//
// This file is part of LastLog.
//
// LastLog is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// LastLog is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LastLog.  If not, see <http://www.gnu.org/licenses/>.

package edu.self.lastLog;

import java.util.Arrays;
import java.util.Comparator;
import org.bukkit.ChatColor;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.CommandSender;

public class PlayerList
{
        public class Entry {
                public String name;
                public long time;
                public Entry(String name, long time) {
                        this.name = name;
                        this.time = time;
                }
        }

        private Entry[] playerList;
        private int length;
        static private Comparator<Entry> comparator;

        static {
                comparator = new Comparator<Entry>() {
                        public int compare(Entry a, Entry b) {
                                if (a.time > b.time) return -1;
                                if (a.time < b.time) return 1;
                                return 0;
                        }
                };
        }

        public PlayerList(OfflinePlayer[] playerList, boolean lastlog) {
                int tmplen = 512;
                length = playerList.length;
                while (tmplen < length) tmplen *= 2;
                this.playerList = new Entry[tmplen];
                int i = 0;
                for (OfflinePlayer player : playerList) {
                        long time = (lastlog ? player.getLastPlayed() : player.getFirstPlayed());
                        this.playerList[i++] = new Entry(player.getName(), time);
                }
        }

        public void sort() {
                Arrays.sort(playerList, 0, length, comparator);
        }

        public int getLength() {
                return length;
        }

        public Entry getEntry(int index) {
                if (index >= length) {
                        throw new IndexOutOfBoundsException("PlayerList.playerList");
                }
                return playerList[index];
        }

        public Entry getEntry(String name) {
                for (int i = 0; i < length; ++i) {
                        Entry entry = playerList[i];
                        if (entry.name.equals(name)) return entry;
                }
                return null;
        }

        Entry addEntry(String name) {
                length += 1;
                if (length > playerList.length) {
                        playerList = Arrays.copyOf(playerList, playerList.length * 2);
                }
                Entry entry = new Entry(name, 0);
                playerList[length - 1] = entry;
                return entry;
        }

        public void set(String name, long time) {
                Entry entry = getEntry(name);
                if (entry == null) {
                        entry = addEntry(name);
                }
                entry.time = time;
        }

        public int getPageCount() {
                return ((getLength() - 1) / LastLogPlugin.PAGE_LENGTH) + 1;
        }
        
        public void displayPage(int pageNumber, boolean lastlog, CommandSender sender) {
                if (pageNumber >= getPageCount()) {
                        sender.sendMessage(LastLogPlugin.ERROR_COLOR + "[LastLog] Page " + (pageNumber + 1) + " selected, but only " + getPageCount() + " available!");
                        return;
                }
                sender.sendMessage(LastLogPlugin.HEADER_COLOR + (lastlog ? "Last login" : "First login") + " - " + getLength() + " Players - Page [" + (pageNumber + 1) + "/" + getPageCount() + "]");
                for (int i = 0; i < LastLogPlugin.PAGE_LENGTH; ++i) {
                        int index = pageNumber * LastLogPlugin.PAGE_LENGTH + i;
                        if (index >= getLength() || index < 0) break;
                        String name = getEntry(index).name;
                        long date = getEntry(index).time;
                        sender.sendMessage(LastLogPlugin.DATE_COLOR + new LastLogDate(date) + " " + ChatColor.WHITE + name);
                }
        }
}
